#ifndef LARGEFILEREADERTASK_H
#define LARGEFILEREADERTASK_H

#include <QObject>
#include <QRunnable>
#include <QFile>
#include <QAtomicInt>

static const quint64 DEFAULT_CHUNK_SIZE_KB = 10000;
static const quint64 MIN_CHUNK_SIZE_KB = 1000;

class LargeFileReaderTask : public QObject, public QRunnable {
    Q_OBJECT
public:
    enum ResultCode {
        RESULT_OK,
        RESULT_FAILED,
        RESULT_CANCELLED
    };

public:
    LargeFileReaderTask( const QString& fileName, quint64 chunkSizeKb = DEFAULT_CHUNK_SIZE_KB );
    ~LargeFileReaderTask();

    void run();

public slots:
    void cancel();

signals:
    void progressChanged( int doneCount, int sumCount );
    void readingFinished( int resultCode, const QByteArray& data = QByteArray() );

private:
    QFile m_file;
    quint64 m_chunkSizeKb;

    QAtomicInt m_cancelledMarker;

};

#endif // LARGEFILEREADERTASK_H
