#include "largefilereadingdemowidget.h"
#include "ui_largefilereadingdemowidget.h"

#include <QFileDialog>
#include <QThreadPool>
#include <QDebug>

#include "largefilereadertask.h"

LargeFileReadingDemoWidget::LargeFileReadingDemoWidget( QWidget* parent ) :
    QWidget( parent ), ui( new Ui::LargeFileReadingDemoWidget ) {
    ui->setupUi( this );

    connect( ui->bnOpen, SIGNAL( clicked( bool ) ), SLOT( onOpenFile() ) );

    ui->bnCancel->setDisabled( true );
}

LargeFileReadingDemoWidget::~LargeFileReadingDemoWidget() {
    emit cancelTaskRequired();

    delete ui;
    ui = NULL;
}

void LargeFileReadingDemoWidget::onOpenFile() {
    QString fileName = QFileDialog::getOpenFileName( this, "Open some large file" );
    if( fileName.isEmpty() ) {
        return;
    }

    LargeFileReaderTask* task = new LargeFileReaderTask( fileName );
    connect( task, SIGNAL( progressChanged( int, int ) ), SLOT( onProgress( int, int ) ) );
    connect( task, SIGNAL( readingFinished( int, QByteArray ) ), SLOT( onReadingFinished( int, QByteArray ) ) );

    connect( ui->bnCancel, SIGNAL( clicked( bool ) ), task, SLOT( cancel() ) );
    connect( this, SIGNAL( cancelTaskRequired() ), task, SLOT( cancel() ) );

    QThreadPool::globalInstance()->start( task );

    ui->bnCancel->setEnabled( true );
    ui->bnOpen->setDisabled( true );
}

void LargeFileReadingDemoWidget::onProgress( int doneCount, int sumCount ) {
    if( ui ) {
        ui->prgBar->setMaximum( sumCount );
        ui->prgBar->setValue( doneCount );
    }
}

void LargeFileReadingDemoWidget::onReadingFinished( int resultCode, const QByteArray& data ) {
    Q_UNUSED( data )

    if( ui ) {
        ui->bnCancel->setDisabled( true );
        ui->bnOpen->setEnabled( true );
    }

    switch ( resultCode ) {
    case LargeFileReaderTask::RESULT_OK: {
        // Успех
        qDebug() << "OK";
        break;
    }

    case LargeFileReaderTask::RESULT_FAILED:
        // Неудача
        qDebug() << "FAILED";
        break;

    case LargeFileReaderTask::RESULT_CANCELLED:
        // Чтение отменено
        qDebug() << "CANCELLED";
        break;

    default:
        break;
    }
}
